module SoarAnalytics
  class ThresholdReporter < Reporter
    def initialize(name:, info_threshold: 0, warn_threshold: nil, error_threshold: nil)
      super(name)
      @thresholds = [
        { 'level' => :error, 'threshold' => error_threshold},
        { 'level' => :warn,  'threshold' => warn_threshold},
        { 'level' => :info,  'threshold' => info_threshold}
      ]
    end

    def report(value, flow_identifier = nil)
      audit_level = determine_highest_matching_auditing_level(value)
      super(simplify_numbers(value), audit_level, flow_identifier) if audit_level
    end

    private

    def determine_highest_matching_auditing_level(value)
      @thresholds.each { | threshold |
        return threshold['level'] if threshold['threshold'] and (value >= threshold['threshold'])
      }
      return nil
    end

    def simplify_numbers(value)
      return "%.6f" % value if value.is_a?(Float)
      return value
    end
  end
end
