require 'soar_auditing_format'
require 'soar_auditing_provider'
require 'soar_flow'
require 'log4r_auditor'

require 'time'
require 'securerandom'
require "soar_analytics"

class Main
  AUDITING_CONFIGURATION = {
    'auditing' => {
      'provider' => 'SoarAuditingProvider::AuditingProvider',
      'level' => 'info',
      'install_exit_handler' => 'true',
      'queue_worker' => {
        'queue_size' => 1000,
        'initial_back_off_in_seconds' => 1,
        'back_off_multiplier' => 2,
        'back_off_attempts' => 5
      },
      'default_nfrs' => {
        'accessibility' => 'local',
        'privacy' => 'not encrypted',
        'reliability' => 'instance',
        'performance' => 'high'
      },
      'auditors' => {
        'log4r' => {
          'adaptor' => 'Log4rAuditor::Log4rAuditor',
          'file_name' => 'soar_sc.log',
          'standard_stream' => 'stdout',
          'nfrs' => {
            'accessibility' => 'local',
            'privacy' => 'not encrypted',
            'reliability' => 'instance',
            'performance' => 'high'
          }
        }
      }
    }
    } unless defined? AUDITING_CONFIGURATION; AUDITING_CONFIGURATION.freeze

    def setup_analytics
      myauditing = SoarAuditingProvider::AuditingProvider.new( AUDITING_CONFIGURATION['auditing'] )
      myauditing.instance_flow_identifier = SoarFlow::ID::generate_flow_id
      myauditing.service_identifier = 'my-test-service.com'
      SoarAnalytics.auditing = myauditing
    end

    def test_time_delta_reporter
      analytic_timer = SoarAnalytics::TimeDeltaReporter.new(name: "mytimer_that_should_not_be_audited", info_threshold: 1, warn_threshold: 2, error_threshold: 3)
      sleep(0)
      analytic_timer.report(SoarFlow::ID::generate_flow_id)

      analytic_timer = SoarAnalytics::TimeDeltaReporter.new(name: "mytimer_with_default_values")
      sleep(0)
      analytic_timer.report(SoarFlow::ID::generate_flow_id)

      analytic_timer = SoarAnalytics::TimeDeltaReporter.new(name: "mytimer", info_threshold: 1, warn_threshold: 2, error_threshold: 3)
      sleep(1)
      analytic_timer.report(SoarFlow::ID::generate_flow_id)

      analytic_timer = SoarAnalytics::TimeDeltaReporter.new(name: "mytimer", info_threshold: 1, warn_threshold: 2, error_threshold: 3)
      sleep(2)
      analytic_timer.report(SoarFlow::ID::generate_flow_id)

      analytic_timer = SoarAnalytics::TimeDeltaReporter.new(name: "mytimer", info_threshold: 1, warn_threshold: 2, error_threshold: 3)
      sleep(3)
      analytic_timer.report(SoarFlow::ID::generate_flow_id)

      flow_id = SoarFlow::ID::generate_flow_id
      reporter = SoarAnalytics::Reporter.new('my_reporter')
      value = 123
      reporter.report(value,:warn,flow_id)

      flow_id = SoarFlow::ID::generate_flow_id
      threshold_reporter = SoarAnalytics::ThresholdReporter.new(name: 'my_threshold_reporter', info_threshold: 1, warn_threshold: 2, error_threshold: 3)
      value = 2
      threshold_reporter.report(value,flow_id)

      flow_id = SoarFlow::ID::generate_flow_id
      timer = SoarAnalytics::TimeDeltaReporter.new(name: "db_access_time", info_threshold: 1, warn_threshold: 2, error_threshold: 3)
      sleep(1)
      timer.report(flow_id)
    end
  end

  main = Main.new
  main.setup_analytics
  main.test_time_delta_reporter
